@@ -5,7 +5,6 @@ import torch
 import torch.nn.functional as F
 from torch import nn, einsum
 
-from einops.layers.torch import Rearrange, Reduce
 from einops import rearrange
 
 from g_mlp_gpt.reversible import ReversibleSequence, SequentialSequence
@@ -87,7 +86,8 @@ class CausalSGU(nn.Module):
         dim,
         dim_seq,
         init_eps = 1e-3,
-        heads = 4
+        heads = 4,
+        act = nn.Identity()
     ):
         super().__init__()
         dim_out = dim // 2
@@ -129,7 +129,8 @@ class CausalLocalSGU(nn.Module):
         dim_seq,
         init_eps = 1e-3,
         heads = 4,
-        window = 128
+        window = 128,
+        act = nn.Identity()
     ):
         super().__init__()
         dim_out = dim // 2
@@ -186,7 +187,7 @@ class AxiallyFold(nn.Module):
             return self.fn(x)
 
         n = x.shape[1]
-        x = pad_to_multiple(x, self.every, dim = 1)
+        x = pad_to_multiple(x, self.every, dim = -2)
         x = rearrange(x, 'b (n e) d -> (b e) n d', e = every)
         x = self.fn(x)
 
@@ -203,14 +204,15 @@ def gMLPBlock(
     dim_ff,
     heads = 4,
     causal = False,
-    window = None
+    window = None,
+    act = nn.Identity()
 ):
     SGU = partial(CausalLocalSGU, window = window) if exists(window) and window < seq_len else CausalSGU
 
     return nn.Sequential(
         nn.Linear(dim, dim_ff),
         nn.GELU(),
-        SGU(dim_ff, seq_len, causal, heads = heads),
+        SGU(dim_ff, seq_len, causal, heads = heads, act = act),
         nn.Linear(dim_ff // 2, dim)
     )
 
@@ -228,7 +230,8 @@ class gMLPGPT(nn.Module):
         ff_mult = 4,
         prob_survival = 1.,
         reversible = False,
-        window = None
+        window = None,
+        act = nn.Identity()
     ):
         super().__init__()
         dim_ff = dim * ff_mult
@@ -239,11 +242,12 @@ class gMLPGPT(nn.Module):
 
         window = cast_tuple(window, depth)
         window = tuple(map(lambda t: t if isinstance(t, tuple) else (t, 1), window))
+        assert len(window) == depth, f'num window sizes {len(window)} must be equal to depth {depth}'
 
         layers = nn.ModuleList([])
 
         for ind, (w, ax) in zip(range(depth), window):
-            get_gmlp = lambda: PreNorm(dim, AxiallyFold(dim, ax, gMLPBlock(dim = dim, dim_ff = dim_ff, seq_len = seq_len, heads = heads, window = w)))
+            get_gmlp = lambda: PreNorm(dim, AxiallyFold(dim, ax, gMLPBlock(dim = dim, dim_ff = dim_ff, seq_len = seq_len, heads = heads, window = w, act = act)))
 
             layer_blocks = nn.ModuleList([
                 get_gmlp()
